/***************************************************************************
*  PCRSAT : a ICOM PCR1000 receiver control                                *
****************************************************************************
* Copyright: i3vfj Vittorio BENVENUTI & iw3fol Vincenzo MEZZALIRA          *
****************************************************************************
* Project start:  july 2000 --- last update:  march 2002                   *
****************************************************************************
* This program permits full control of ICOM PCR1000 all mode receiver,     *
* and  by socket connection with PREDICT (kd2bd) tracks the doppler on     *
* satellite downlink. Also has graphical signal meter (uncalibrated) with  *
* 8 bit range and autoscale feature ( '^' 0/64 range and '*' 65/256 range).*
* Update rate, software selectable, down to 100 msec.                      *
****************************************************************************
* Socket code inspired from KD4HBO (Ivan) demo bundled with PREDICT        *
****************************************************************************
* Some code cames from PREDICT ( a big TNX to John for his effort) to      * 
* mantain the same 'look'n feel' for the users                             *
****************************************************************************
* The ncurses graphics routine are from Vincenzo IW3FOL - MANY THANKS      *
****************************************************************************
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation; either version 2 of the License or any later   *
* version.                                                                 *
*                                                                          *
* This program is distributed in the hope that it will be useful,          *
* but WITHOUT ANY WARRANTY; without even the implied warranty of           *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        *
* General Public License for more details.                                 *
*                                                                          *
***************************************************************************/



/* include header file */
#include "my_include.h"

/* globali variables   */
const char vers$[5]={"0.58"};    /* version */
const int query_per_second = 2;  /* query to predict server */
char port [20], command_rx [64];
int fd_serial;
char string[625];
char buf [625];
char sat_db [24][26];

/* include socket related function */
#include "socketfunction.c"

/* include serial related function */
#include "serialfunction.c"

/* include graph/text related function */
#include "graph.c"


/* main */
int main(int argc, char **argv)
{
unsigned int ifsh=0x80,volu=0x4f,sque=0x01,sel=0, mod=1;
unsigned long frequency=435100000, step=10;
const char filt_d[6][6] = {"    3","    6","  1 5","  5 0","2 3 0"};
const unsigned int filt_s[6] = {0,1,2,3,4};
const char mode_d[7][6] = {"L S B","U S B"," A M "," C W "," N F ","W F M"};
const unsigned int mode_s[7] = {0,1,2,3,5,6};
char valore[15],DATA[30],DATA_SET[30],DISP[16],cam[7];
unsigned int change=1,indice=0;
unsigned int agc=0,nb=0,att=0;
unsigned int uplink_idx=0,trasponder=1;
unsigned int stop=0;
unsigned int command=0;
unsigned int sat_track=0, trk=0, doppler=0;
unsigned long fx=0, fx_uplink=0;  
char host [64] = "localhost";
const char uplink[5][6]={"     ","   21","  146","  435"," 1267"};
struct termios serial_options;
struct termios way_it_was;

initscr();
start_color();
noecho();
scrollok(stdscr,TRUE);
curs_set(0);
intrflush(stdscr, FALSE);
keypad(stdscr, TRUE);
cbreak();
halfdelay(10/query_per_second);

init_pair(1,COLOR_WHITE,COLOR_BLACK);
init_pair(2,COLOR_WHITE,COLOR_BLUE);
init_pair(3,COLOR_YELLOW,COLOR_BLUE);
init_pair(4,COLOR_CYAN,COLOR_BLUE);
init_pair(5,COLOR_WHITE,COLOR_RED);
init_pair(6,COLOR_RED,COLOR_WHITE);
init_pair(7,COLOR_CYAN,COLOR_RED);

bkgdset(COLOR_PAIR(2));
clear();
refresh();

serial_init();
set_serial (fd_serial, &serial_options, &way_it_was);
PCR1000_init1(fd_serial);

/*  serial_close(fd_serial);
    restore_serial(fd_serial, &way_it_was);
    serial_init();
    set_serial_hi(fd_serial, &serial_options, &way_it_was);
*/ 

draw_screen();	

	mvprintw(8,35,mode_d[mod]);
	mvprintw(8,45,filt_d[sel]);     
	mvprintw(14,69,"   "); 
	mvprintw(14,69,"%i",ifsh); 
	mvprintw(22,69,"   ");
	mvprintw(22,69,"%i",sque);
	mvprintw(18,69,"   ");
	mvprintw(18,69,"%i",volu);
        mvprintw(8,15,"%10ld",frequency);  
        mvprintw(8,66,"%8ld",step);

for(;;)
{
	        sprintf(cam,"%s%02X","J43",ifsh);
		serial_out(fd_serial,cam); 
		sprintf(cam,"%s%02X","J40",volu);
		serial_out(fd_serial,cam); 
		sprintf(DATA_SET,"%s%010ld%s%d%s%d%s","K0",frequency,"0",mode_s[mod],"0",filt_s[sel],"00");
		serial_out(fd_serial,DATA_SET);
	if (change==1)
		{
		sprintf(DATA_SET,"%s%010ld%s%d%s%d%s","K0",frequency,"0",mode_s[mod],"0",filt_s[sel],"00");
		serial_out(fd_serial,DATA_SET);
		tcflush (fd_serial, TCIFLUSH); 
		change=0;  
		}
	
	command=getch();
	{
	switch(command)
		{
			case     -1: /* no key pressed. Get signal level */
			  get_level(fd_serial);
			if (trk==1)
			{
			  if (trasponder==1) /* get right trasponder constant */
			  {
			   fx_uplink = abs ((frequency - (atol (uplink[uplink_idx]))* 10e5));
			  }
			  else
			  {
			   fx_uplink = ((atol (uplink[uplink_idx]))* 10e5) + frequency;
			  }
			  /* get doppler and track */
			  doppler = ((int) (get_doppler(host,sat_track)))*(fx_uplink * 10e-9);
		         frequency = fx + doppler;
			 sprintf(DATA_SET,"%s%010ld%s%d%s%d%s","K0",frequency,"0",mode_s[mod],"0",filt_s[sel],"00");
        		 mvprintw(8,15,"%10ld",frequency);  
			 mvprintw(19,31," %6i Hz ",doppler);  
			 serial_out(fd_serial,DATA_SET);
			 tcflush (fd_serial, TCIFLUSH);
		         get_sat_pos(host,sat_track);
			}
			
			break;
		case     27:      /* ESC exit */
			endwin();
			stop=1;
			serial_out(fd_serial,"H100\x0D\x0A");
			serial_close( fd_serial);
			restore_serial( fd_serial,  &way_it_was);
			exit(0);
			break;
		case	KEY_RIGHT:      /* RightArrow frequency up */
			frequency=frequency+step;
			change=1;
			if (frequency > 1300000000)
				frequency=100000;
				if (trk==1)
				{
				   fx = frequency - doppler;
				}
        		mvprintw(8,15,"%10ld",frequency);  
			break;
		case	KEY_LEFT:     /* LeftArrow frequency down */
			frequency=frequency-step;
			change=1;
			if (frequency < 100000)
				frequency=1300000000;
				if (trk==1)
				{
				   fx = frequency - doppler;
				}
        		mvprintw(8,15,"%10ld",frequency);  
			break;
		case	KEY_F(1):    /* F1 ifbw */
			sel++;
			change=1;
			if (sel>4)
				sel=0;
				mvprintw(8,45,filt_d[sel]);     
			break;
		case	KEY_F(2):	/* F2 mode */
			mod++;
			change=1;
			if (mod>5)
				mod=0;
				mvprintw(8,35,mode_d[mod]);
			break;
		case	KEY_F(3):    /* F3 noise blanker */
			change=1; 
			if (nb==1)
				{
				nb=0;
				serial_out(fd_serial,"J4600\n\r");
				mvprintw(17,9," ");
				}
			else
				{
				nb=1;
				serial_out(fd_serial,"J4601\n\r");
				mvprintw(17,9,"*");
				}
			break;
		case	KEY_F(4):    /* F5 agc */
			change=1; 
			if (agc==1)
				{
				agc=0;
				serial_out(fd_serial,"J4700\n\r");
                                mvprintw(19,9," ");
				}
			else
				{
				agc=1;
				serial_out(fd_serial,"J4701\n\r");
                                mvprintw(19,9,"*");
				}
			break;
		case 	KEY_F(5):    /* F4 attenuator */
			if (att==1)
				{
				att=0;
				serial_out(fd_serial,"J4500\n\r");
                                mvprintw(21,9," ");
				}
			else
				{
				att=1;
				serial_out(fd_serial,"J4501\n\r");
                                mvprintw(21,9,"*");
				}
			break;
		case 	KEY_F(6):    /* F6 tracking doppler */
			if (trk==1)
				{
				trk=0;
				change = 1;
				sprintf(valore,"%10ld",frequency);
        			mvprintw(8,15,valore);  
                                mvprintw(23,9," ");
				mvprintw(19,32,"                      ");
				}
			else
				{
				trk=1;
                                mvprintw(23,9,"*");
		                mvprintw(19,45,uplink[uplink_idx]);      
				if (trasponder==1)
					{
					mvprintw(19,51,"NOR");
					}
				else
					{
					mvprintw(19,51,"INV");
					}
				doppler = ((int) (get_doppler(host,sat_track)))*(frequency * 10e-9);
				fx = (frequency - doppler); 
				}
			break;
		case    KEY_NPAGE:    /* PageDown select [non]inverting trasponder */
			if (trk==0)
			{
			if (trasponder==1)
				{ 
				trasponder=-1;
				mvprintw(19,51,"INV");
		                mvprintw(19,45,uplink[uplink_idx]);      
				}
			else
				{
				trasponder=1;
				mvprintw(19,51,"NOR");
		                mvprintw(19,45,uplink[uplink_idx]);      
				}
			}
			break;
		case    KEY_PPAGE:    /* ShiftF5 select uplink  */
			if (trk==0)
			{
			uplink_idx++;
			if (uplink_idx >= 5)
				uplink_idx=0;
				if (trasponder==1)
					{
					mvprintw(19,51,"NOR");
					}
				else
					{
					mvprintw(19,51,"INV");
					}
				mvprintw(19,45,uplink[uplink_idx]);      
			}
			break;
		case 	32:    /* ShiftF6 select satellite */
				trk=0;
				mvprintw(23,9," ");
				get_database(host);
				sat_track = Select(); 
			erase(); 
			draw_screen();
			  mvprintw(8,35,mode_d[mod]);
			  mvprintw(8,45,filt_d[sel]);     
			  mvprintw(14,69,"   ");
			  mvprintw(14,69,"%i",ifsh);
			  mvprintw(22,69,"   ");
			  mvprintw(22,69,"%i",sque);
			  mvprintw(18,69,"   ");
	   		  mvprintw(18,69,"%i",volu);
        		  mvprintw(8,15,"%10ld",frequency);  
			  mvprintw(13,48,"%s",sat_db[sat_track]);  
        		  mvprintw(8,66,"%8ld",step);
		          mvprintw(19,45,uplink[uplink_idx]);      
				if (trasponder==1)
					{
					mvprintw(19,51,"NOR");
					}
				else
					{
					mvprintw(19,51,"INV");
					}
			refresh();
			break;
		case	KEY_F(7):    /* F7 if shift up */
			ifsh++;
			if (ifsh>0xFE)
				ifsh=0xFF;
			sprintf(cam,"%s%02X","J43",ifsh);
			serial_out(fd_serial,cam);
				indice=(abs(ifsh/20));
				strcpy(DISP,"-------------");
				DISP[indice]='|';
				mvprintw(10,65,DISP);
				  mvprintw(14,69,"   ");
	  			  mvprintw(14,69,"%i",ifsh);
			break;
		case	KEY_F(8):	/* F8 if shift down */
			ifsh--;
			if (ifsh<0x01)
				ifsh=0x01;
			sprintf(cam,"%s%02X","J43",ifsh);
			serial_out(fd_serial,cam);
				indice=(abs(ifsh/20));
				strcpy(DISP,"-------------");
				DISP[indice]='|';
				mvprintw(10,65,DISP);
				  mvprintw(14,69,"   ");
				  mvprintw(14,69,"%i",ifsh);
			break;
		case	'r':	/* reset if shift to center */
			ifsh=0x7F;
			sprintf(cam,"%s%02X","J43",ifsh);
			serial_out(fd_serial,cam);
				indice=(abs(ifsh/20));
				strcpy(DISP,"-------------");
				DISP[indice]='|';
				mvprintw(10,65,DISP);
				  mvprintw(14,69,"   ");
	  			  mvprintw(14,69,"%i",ifsh);
			break;
		case	KEY_F(9):    /* F9 volume up */
			volu++;
			if (volu>0xFE)
				volu=0xFF;
			sprintf(cam,"%s%02X","J40",volu);
			serial_out(fd_serial,cam);
			  mvprintw(18,69,"   ");
			  mvprintw(18,69,"%i",volu);
			break;
		case	KEY_F(10):	/* F10 volume down */
			volu--;
			if (volu<0x01)
				volu=0x01;
			sprintf(cam,"%s%02X","J40",volu);
			serial_out(fd_serial,cam);
			  mvprintw(18,69,"   ");
			  mvprintw(18,69,"%i",volu);
			break;
		case	KEY_F(11):    /* F11 squelch up*/
			sque++;
			if (sque>0xFE)
				sque=0xFF;
			sprintf(cam,"%s%02X","J41",sque);
			serial_out(fd_serial,cam);
			  mvprintw(22,69,"   ");
			  mvprintw(22,69,"%i",sque);
			break;
		case	KEY_F(12):	/* F12 squelch down */
			sque--;
			if (sque<0x01)
				sque=0x01;
			sprintf(cam,"%s%02X","J41",sque);
			serial_out(fd_serial,cam);
			  mvprintw(22,69,"   ");
			  mvprintw(22,69,"%i",sque);
			break;
		case	KEY_UP:    /* UpArrow fx step up */
			step=step*10;
			if (step > 10000000)
					step=1;
			sprintf(DATA,"%8ld",step);
                        mvprintw(8,66,DATA);
			break;
		case	KEY_DOWN:    /* DownArrow fx step down */
			step=step/10;
			if (step < 1)
				step=10000000;
			sprintf(DATA,"%8ld",step);
                        mvprintw(8,66,DATA);
			break;
		default:  break;
			}
		}
	}
}
