void handler()
{
	/* This is a function that is called when the response function
	   times out.  This is in case the server fails to respond. */

	signal(SIGALRM,handler);
}


int connectsock(char *host, char *service, char *protocol)
{
	/* This function is used to connect to the server.  "host" is the
	   name of the computer on which PREDICT is running in server mode.
	   "service" is the name of the socket port.  "protocol" is the
	   socket protocol.  It should be set to UDP. */

	struct hostent *phe;
	struct servent *pse;
	struct protoent *ppe;
	struct sockaddr_in sin;
	
	int s, type;
	
	bzero((char *)&sin,sizeof(struct sockaddr_in));
	sin.sin_family=AF_INET;
	
	if ((pse=getservbyname(service,protocol)))
		sin.sin_port=pse->s_port;

	else if ((sin.sin_port=htons((unsigned short)atoi(service)))==0)
	{
		/* printf("Can't get services\n"); */
		mvprintw(15,29,"Can't get services"); 
		return -1;
	}

	if ((phe=gethostbyname(host)))
		bcopy(phe->h_addr,(char *)&sin.sin_addr,phe->h_length);

	else if ((sin.sin_addr.s_addr=inet_addr(host))==INADDR_NONE)
	{
		/* printf("Can't get host.\n"); */
		mvprintw(15,29,"Can't get host"); 
		return -1;
	}
	
	if ((ppe=getprotobyname(protocol))==0)
		return -1;

	if (strcmp(protocol,"udp")==0)
		type=SOCK_DGRAM;
	else
		type=SOCK_STREAM;
	
	s=socket(PF_INET,type,ppe->p_proto);

	if (s<0)
	{
		/* printf("Can't get socket.\n"); */
		mvprintw(15,29,"Can't get socket"); 
		return -1;
	}

	if (connect(s,(struct sockaddr *)&sin,sizeof(sin))<0)
	{
		/* printf("Can't connect to socket.\n"); */
		mvprintw(15,29,"Can't connect to socket"); 
		return -1;
	}

	return s;
}


void get_response(int sock, char *buf)
{
	/* This function gets a response from the
	   server in the form of a character string. */

	int n;
	alarm(2); 
	n=read(sock,buf,625);

	if (errno==EINTR)
		return;

	if (n<0)
	{
		alarm(0); 
		return;
	}
	
	buf[n]='\0';
	alarm(0); 
}

char *send_command(host, command)
char *host, *command;
{
	int sk;

	/* This function sends "command" to PREDICT running on
	   machine "host", and returns the result of the command
	   as a pointer to a character string. */

	/* Open a network socket */
	sk=connectsock(host,"predict","udp");

	if (sk<0)
		exit(-1);

	/* Build a command buffer */
	sprintf(string,"%s\n",command);

	/* Send the command to the server */
   	write(sk,command,strlen(string));

	/* clear string[] so it can be re-used for the response */
	string[0]=0;

	/* Get the response */
   	get_response(sk,string);

	/* Close the connection */
   	close(sk);

	return string;
}

float get_doppler (char *host, int sat_track)
{
	/* This function get satellite doppler from host running predict */
	
	char cmd [64];
	float doppler = 0.0;
	
	sprintf(cmd,"%s %s","GET_DOPPLER",sat_db[sat_track]);
	strcpy(buf, send_command(host,cmd));
	sscanf(buf,"%f",&doppler);
	
	return (doppler);
}

void get_sat_pos (char *host, int sat_track)
{
	/* this function get satellite position from host running predict */
	
	char cmd[64];
	char satname [10];
	int x,y;
	float slong, slat, az, el, footprint, range, altitude, velocity; 
	long  aostime, orbitnum;
	char visibility;
	
	{
		sprintf(cmd,"GET_SAT %s",sat_db[sat_track]);
		strcpy(buf, send_command(host,cmd));
		/* Parse the response from GET_SAT */
		/* The first element of the response is the satellite name.
		   It is ended with a '\n' character and many contain spaces. */

		for (x=0; buf[x]!='\n'; x++)
			satname[x]=buf[x];

		satname[x]=0;
		x++;

		/* The rest of the data from GET_SAT is numerical, and
		   can be parsed using the sscanf() function.  First, the
		   satellite name is removed from "buf", and then "buf"
		   is parsed for numerical data using an sscanf(). */

		for (y=0; buf[x+y]!=0; y++)
			buf[y]=buf[x+y];

		buf[y]=0;
 
		sscanf(buf,"%f %f %f %f %ld %f %f %f %f %ld %c",&slong,&slat,&az,&el,&aostime,&footprint,&range,&altitude,&velocity,&orbitnum,&visibility);
	}
	 	mvprintw(15,27,"az:%3.1f  ", az );  
	 	mvprintw(15,37,"el:%2.1f  ", el );  
	 	mvprintw(15,46,"range:%5.0f ", range );  
	return ; 

}

void get_database (char *host)
{
	/* This function get satellite database from host running predict */
	
	int x,y,z;  
	char cmd [64] = "GET_LIST";
	
	strcpy(buf, send_command(host,cmd));
	  	
	for (x=0, y=0, z=0; y<strlen(buf); y++)
	{
		if (buf[y]!='\n')
		{
			sat_db[z][x]=buf[y];
			x++;
		}

		else
		{
			sat_db[z][x]=0;
			z++;
			x=0;
		}
	}

	mvprintw (19,36,"Letti %i sat",z);      
	return;
}

int Select()
{
	/* This function displays the names of satellites contained
	   within the program's database and returns an index that
	   corresponds to the satellite selected by the user.  An
	   ESC or CR returns a -1. */

	int x, y, z, key=0;

	clear();
	
	/* bkgdset(COLOR_PAIR(2)|A_BOLD);  */ 
	printw("\n\n\t\t\t      Select a Satellite:\n\n");

	/* attrset(COLOR_PAIR(3)|A_BOLD); */ 

	for (x=0, y=8, z=16; y<16; ++x, ++y, ++z)
	{
		printw("\n\t[%c]: %-15s", x+'A', sat_db[x]);
		printw("\t[%c]: %-15s", y+'A', sat_db[y]);
		printw("\t[%c]: %-15s\n", z+'A', sat_db[z]);
	} 

	/* attrset(COLOR_PAIR(4)|A_BOLD); */

	printw("\n\n\t\t<< Enter Selection  -  Press [ESC] To Exit >>");
	refresh();

	do
	{
		key=toupper(getch());

		if (key==27 || key=='\n')
			return (-1);
	}
	while (key<'A' || key>'X');

	return(key-'A'); 
}

